using System;
using System.IO;
using System.Text;
using com.swivelsecure.pinsafe;

// ReSharper disable CheckNamespace

namespace SwivelSecure.OWA2007.Install
	// ReSharper restore CheckNamespace
{
	internal class ModifyLogon {
		private static int Main(string[] args) {
			string folder = Directory.GetCurrentDirectory();

			string configFile = folder + "\\web.config";

			PINsafeSettings settings = new PINsafeSettings();
			if (File.Exists(configFile)) {
				settings.LoadSettings(configFile);
				Debug.WriteLine("Loaded settings from " + configFile);
			} else {
				settings.SetDefaults();
				Debug.WriteLine("Loaded default settings");
			}

			string filename = null;
			if (args != null && args.Length > 0) {
				if (File.Exists(args[0])) {
					// This is an absolute path.
					filename = args[0];
				} else {
					filename = folder + (args[0].StartsWith("\\") ? "" : "\\") + args[0];
				}
			}
			if (string.IsNullOrEmpty(filename)) {
				filename = folder + "\\auth\\Logon.aspx";
			}

			if (!File.Exists(filename)) {
				Console.Error.WriteLine("File " + filename + " does not exist");
				Debug.WriteLine("File " + filename + " does not exist");
				return 2;
			}
			string content;
			using (StreamReader reader = new StreamReader(filename)) {
				content = reader.ReadToEnd();
			}
			StringBuilder output = new StringBuilder();

			bool usingBackup = false;
			string backupName = filename + ".sav";
			// First, check if the customisation has already been done.
			int index = content.IndexOf("pinsafe", StringComparison.OrdinalIgnoreCase);
			if (index > 0) {
				// If it has, find the backup file.
				if (!File.Exists(backupName)) {
					// If the backup file doesn't exist, fail.
					Console.Error.WriteLine("Page has already been customised, and no backup exists.");
					Debug.WriteLine("Page has already been customised, and no backup exists.");
					return 5;
				}
				// OK, reload content from the backup file.
				using (StreamReader reader = new StreamReader(backupName)) {
					content = reader.ReadToEnd();
				}
				usingBackup = true;
				Debug.WriteLine("Creating customised logon using backup of original login page");
			} else {
				Debug.WriteLine("Creating customised logon page");
			}
			// Find the first import statement.
			index = content.IndexOf("<%@ Import", StringComparison.Ordinal);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<%@ Import\"");
				return 3;
			}
			output.Append(content.Substring(0, index));
			output.Append("<%@ Import namespace=\"com.swivelsecure.pinsafe\"%>\r\n");
			int curpos = index;
			// Now skip over the !ReplaceCurrent part of the page.
			index = content.IndexOf("</html>", curpos, StringComparison.OrdinalIgnoreCase);
			// Now find the end of the header.
			index = content.IndexOf("</head>", index, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"</head>\"");
				return 3;
			}
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			// Insert the javascript references
			output.Append("<script type=\"text/javascript\" src=\"pinsafe.js\"></script>\r\n");
			// Find the body element
			index = content.IndexOf("<body", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<body\"");
				return 3;
			}
			index += 5;
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			// Insert the onload attribute
			output.Append(" onload=\"CheckClient();\"");
			// Find the form attribute
			index = content.IndexOf("<form", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<form\"");
				return 3;
			}
			index = content.IndexOf("action=", index, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Console.Error.WriteLine("Unable to find expected text: \"action=\"");
				return 3;
			}
			index += 7;
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			// Replace the form action.
			index = content.IndexOf(content[curpos], curpos + 1);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: end-quote");
				return 3;
			}
			curpos = index + 1;
			output.Append("\"PINsafeLogon.aspx\"");
			// Find the username input box.
			index = content.IndexOf("<input id=\"username\"", StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<input id=username\"");
				return 3;
			}
			index = content.IndexOf(">", index, StringComparison.Ordinal);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: end of username field");
				return 3;
			}
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			// Insert the onblur event for the username.
			output.Append(" onblur=\"CheckUser();\"");
			// Find the password row.
			index = content.IndexOf("<input id=\"password\"", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<input id=password\"");
				return 3;
			}
			index = content.IndexOf("<tr>", index, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<tr>\"");
				return 3;
			}
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			output.Append("<tr id=\"OTCRow\" style=\"display:none\">\r\n");
			output.Append("<td nowrap><label for=\"otc\">One-Time Code:</label></td>\r\n");
			output.Append("<td class=\"txtpad\">\r\n");
			output.Append("<input id=\"otc\" name=\"otc\" type=\"password\" class=\"txt\">\r\n");
			output.Append("<input type=\"checkbox\" name=\"noPINsafe\" id=\"noPINsafe\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"allowUnknown\" id=\"allowUnknown\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"userExists\" id=\"userExists\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"showTuring\" id=\"showTuring\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"showMessage\" id=\"showMessage\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"autoShow\" id=\"autoShow\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"showPinpad\" id=\"showPinpad\" style=\"display:none\"/>\r\n");
			output.Append("<input type=\"checkbox\" name=\"showChangePin\" id=\"showChangePin\" style=\"display:none\"/>\r\n");
			output.Append("</td></tr>\r\n");
			// Find the cell containing the LogOn button
			index = content.IndexOf("align=\"right\"", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"align=right\"");
				return 3;
			}
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index + 14; // Skip the align=right attribute
			index = content.IndexOf(">", curpos, StringComparison.Ordinal);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected end of tr tag");
				return 3;
			}
			// Copy the rest of the td element.
			output.Append(content.Substring(curpos, index - curpos + 1));
			curpos = index + 1;
			// Insert a table inside this cell.
			output.Append("<table width=\"100%\"><tr>\r\n");
			// Insert the cell containing the image request button.
			output.Append("<td>");
			output.Append("<span id=\"TuringButton\" style=\"display:none\">");
			output.Append("<input type=\"button\" class=\"btn\" value=\"New Security String\" onclick=\"ShowTuring()\" />");
			output.Append("</span></td>\r\n");
			output.Append("<td><span id=\"MessageButton\" style=\"display:none\">");
			output.Append("<input type=\"button\" class=\"btn\" value=\"New Message\" onclick=\"ShowMessage()\" />");
			output.Append("</span></td>\r\n");
			// Find the end of this cell.
			index = content.IndexOf("</td>", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"</td>\"");
				return 3;
			}
			// Insert the new cell start
			output.Append("<td width=\"100%\" align=\"right\">");
			// Insert the original cell contents
			output.Append(content.Substring(curpos, index - curpos));
			// Insert the end of the table
			output.Append("</td></tr></table>\r\n");
			curpos = index;
			// Find the end of the current table.
			index = content.IndexOf("</table>", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"</table>\"");
				return 3;
			}
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			// Insert the TURing row.
			output.Append("<tr id=\"TuringRow\" style=\"display:none\">\r\n");
			output.Append("<td colspan=\"2\" align=\"center\">\r\n");
			output.Append("<img id=\"TuringImage\" src=\"SCImage.aspx\" /></td></tr>\r\n");
			// Insert the Pinpad row
			output.Append("<tr id=\"PinpadRow\" style=\"display:none\">\r\n");
			output.Append("<td colspan=\"2\" align=\"center\">\r\n");
			output.Append("<table id=\"tblPinpad\">\r\n");
			output.Append("<tr id=\"rowPinpad1\">\r\n");
			output.Append("<td id=\"cellPinpad1\"><img id=\"imgPinpad1\" onclick=\"addDigit('1')\" /></td>\r\n");
			output.Append("<td id=\"cellPinpad2\"><img id=\"imgPinpad2\" onclick=\"addDigit('2')\" /></td>\r\n");
			output.Append("<td id=\"cellPinpad3\"><img id=\"imgPinpad3\" onclick=\"addDigit('3')\" /></td>\r\n");
			output.Append("</tr>\r\n");
			output.Append("<tr ID=\"rowPinpad2\">\r\n");
			output.Append("<td id=\"cellPinpad4\"><img id=\"imgPinpad4\" onclick=\"addDigit('4')\" /></td>\r\n");
			output.Append("<td id=\"cellPinpad5\"><img id=\"imgPinpad5\" onclick=\"addDigit('5')\" /></td>\r\n");
			output.Append("<td id=\"cellPinpad6\"><img id=\"imgPinpad6\" onclick=\"addDigit('6')\" /></td>\r\n");
			output.Append("</tr>\r\n");
			output.Append("<tr ID=\"rowPinpad3\">\r\n");
			output.Append("<td id=\"cellPinpad7\"><img id=\"imgPinpad7\" onclick=\"addDigit('7')\" /></td>\r\n");
			output.Append("<td id=\"cellPinpad8\"><img id=\"imgPinpad8\" onclick=\"addDigit('8')\" /></td>\r\n");
			output.Append("<td id=\"cellPinpad9\"><img id=\"imgPinpad9\" onclick=\"addDigit('9')\" /></td>\r\n");
			output.Append("</tr>\r\n");
			output.Append("<tr ID=\"rowPinpad0\">\r\n");
			output.Append("<td>&nbsp;</td>\r\n");
			output.Append("<td id=\"cellPinpad0\"><img id=\"imgPinpad0\" onclick=\"addDigit('0')\" /></td>\r\n");
			output.Append("<td>&nbsp;</td>\r\n");
			output.Append("</tr>\r\n");
			output.Append("<tr><td colspan=\"3\" align=\"center\"><input type=\"button\" class=\"btn\" value=\"Refresh\" onclick=\"ShowPinpad()\" /></td></tr>");
			output.Append("</table></td></tr>\r\n");
			//
			index = content.IndexOf("<table class=\"mid tblConn\"", curpos, StringComparison.OrdinalIgnoreCase);
			if (index < 0) {
				Console.Error.WriteLine("Unable to find expected text");
				Debug.WriteLine("Unable to find expected text: \"<table class=\"mid tblConn\"\"");
				return 3;
			}
			output.Append(content.Substring(curpos, index - curpos));
			curpos = index;
			// Insert the Change PIN link
			output.Append("<table class=\"mid\" id=\"tblChangePin\" style=\"display:none\"><tr><td align=\"right\">");
			output.Append("<a href=\"./ChangePIN.aspx?redirect=./logon.aspx\">Change PIN</a>");
			output.Append("</td></tr></table>\r\n");
			// That's all: append the rest of the file and save it.
			output.Append(content.Substring(curpos));
			string outfile = filename + ".tmp";
			using (StreamWriter writer = new StreamWriter(outfile)) {
				try {
					writer.Write(output);
				} catch (IOException) {
					Console.Error.WriteLine("Error writing modified text");
					Debug.WriteLine("Error writing modified text");
					return 4;
				}
			}
			try {
				if (usingBackup) {
					// Rename the current logon page to ".old"
					string oldName = filename + ".old";
					File.Move(filename, oldName);
				} else {
					if (File.Exists(backupName)) {
						File.Delete(backupName);
					}
					File.Move(filename, backupName);
				}
				File.Move(outfile, filename);
			} catch (IOException x) {
				Console.Error.WriteLine("Error renaming backup file: " + x.Message);
				Debug.WriteLine("Error renaming backup file: " + x.Message);
				return 5;
			}
			return 0;
		}
	}
}