using System;
using System.IO;
using System.Windows.Forms;
using com.swivelsecure.pinsafe;
using Microsoft.Win32;
using swivelsecure.pinsafe.config.Properties;

namespace swivelsecure.pinsafe.config {
    internal static class Program {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        private static int Main(string[] args) {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            string owaLocation = null;
            // Check the location from the registry
            try {
                using (
                    RegistryKey regRootKey = Registry.LocalMachine.CreateSubKey(@"SOFTWARE\Microsoft\ExchangeServer\v14")
                    ) {
                    if (regRootKey != null) {
                        using (RegistryKey setupKey = regRootKey.CreateSubKey("Setup")) {
                            if (setupKey != null) {
                                object path = setupKey.GetValue("MsiInstallPath");
                                owaLocation = (path == null ? Settings.Default.OWALocation : path.ToString());
                            }
                        }
                    }
                }
            } catch {
                // OK, can't be bothered.
            }
            if (owaLocation == null) {
                owaLocation = Settings.Default.OWALocation;
            }
            if (owaLocation.EndsWith("\\")) {
                owaLocation = owaLocation + "ClientAccess\\Owa";
            } else {
                owaLocation = owaLocation + "\\ClientAccess\\Owa";
            }
            if (!Directory.Exists(owaLocation)) {
                FolderBrowserDialog dlg = new FolderBrowserDialog {
                                                                      Description = "Please locate the OWA root folder"
                                                                  };
                if (dlg.ShowDialog() == DialogResult.OK) {
                    owaLocation = dlg.SelectedPath;
                }
            }
            bool run = true;
            if (args.Length > 0) {
                char opt = Char.ToLower(args[0][0]);
                if (opt == '-' && args[0].Length > 1) {
                    opt = Char.ToLower(args[0][1]);
                }
                switch (opt) {
                    case 'i': {
                        // Install default settings if none there already.
                        PINsafeSettings settings = new PINsafeSettings(false);
                        if (!settings.LoadSettings(owaLocation + "\\web.config")) {
                            settings.SetDefaults();
                            settings.SaveSettings(owaLocation + "\\web.config");
                        }
                        run = false;
                    }
                        break;
                    case 'u': {
                        // Remove PINsafe settings
                        PINsafeSettings settings = new PINsafeSettings(false);
                        settings.RemoveSettings(owaLocation + "\\web.config");
                        // Rename the old logon page.
                        string originalLogonPage = owaLocation + @"\auth\logon.aspx.sav";
                        if (File.Exists(originalLogonPage)) {
                            string currentLogonPage = owaLocation + @"\auth\logon.aspx";
                            if (File.Exists(currentLogonPage)) {
                                File.Delete(currentLogonPage);
                            }
                            File.Move(originalLogonPage, currentLogonPage);
                        }
                        run = false;
                    }
                        break;
                }
            }
            if (run) {
                ConfigForm form = new ConfigForm {OwaLocation = owaLocation};
                Application.Run(form);
            }
            return 1;
        }
    }
}