﻿using System.Collections.Generic;
using System.Windows.Forms;
using com.swivelsecure.pinsafe;

namespace swivelsecure.pinsafe.config {
    public partial class ServersForm : Form {

        private readonly ConfigForm parentForm;
        private bool initializing;

        public ServersForm(ConfigForm parent) {
            InitializeComponent();
            parentForm = parent;
        }

        public void SetList(List<ServerSettings> servers, int activeId) {
            initializing = true;
            lstServers.Items.Clear();
            foreach (ServerSettings server in servers) {
                lstServers.Items.Add(server.Address);
            }
            lstServers.SelectedIndex = activeId;
            initializing = false;
        }

        private void lstServers_SelectedIndexChanged(object sender, System.EventArgs e) {
            btnDel.Enabled = (lstServers.SelectedIndex > 0);
            if (initializing) {
                return;
            }
            parentForm.ActiveServerId = lstServers.SelectedIndex;
        }

        private void btnAdd_Click(object sender, System.EventArgs e) {
            parentForm.AddServer();
        }

        private void btnDel_Click(object sender, System.EventArgs e) {
            int selectedIndex = lstServers.SelectedIndex;
            if (selectedIndex > 0) {
                if (MessageBox.Show(this, "Delete server \"" + lstServers.Items[selectedIndex] + "\"?", "Delete PINsafe Server", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) {
                    parentForm.DeleteServer(selectedIndex);
                }
            }
        }
    }
}
