﻿using System;
using System.Web;
using com.swivelsecure.pinsafe;

namespace PINsafeLogin.auth {
    public partial class ChangePIN : System.Web.UI.Page {
        protected void Page_Load(object sender, EventArgs e) {
            if (!IsPostBack) {
                if (!String.IsNullOrEmpty(Request["username"])) {
                    txtUsername.Text = Request["username"];
                }
                if (!String.IsNullOrEmpty(Request["redirect"])) {
                    txtRedirect.Value = Request["redirect"];
                }
				if (!String.IsNullOrEmpty(Request["reason"])) {
					string reason = txtReason.Value = Request["reason"];
					// Reason = 1 means PIN expired, so you can't skip change PIN.
					btnCancel.Visible = !"1".Equals(reason);
					if ("1".Equals(reason)) {
						lblResult.Text = "Your PIN has expired. Please change it before continuing to OWA";
					} else if ("2".Equals(reason)) {
						lblResult.Text = "Your PIN will expire within the next 24 hours. It is recommended that you change it before continuing to OWA";
					} else if ("3".Equals(reason)) {
						lblResult.Text = "Your PIN will expire shortly. Please consider changing it";
					}
				}
            }
        }

		private void CheckCookies() {
			HttpCookie psCookie = Request.Cookies["PINSAFE"];
			if (psCookie == null) {
				psCookie = Request.Cookies["PINSAFE_CP"];
				if (psCookie != null) {
					// Rename the cookie
					string cookieData = psCookie.Value;
					psCookie = new HttpCookie("PINSAFE", cookieData) {Path = "/"};
					Response.Cookies.Add(psCookie);
					psCookie = new HttpCookie("PINSAFE_CP", "") {Path="/", Expires = DateTime.MinValue};
					Response.Cookies.Add(psCookie);
				}
			}
		}

        protected void btnLogin_Click(object sender, EventArgs e) {
	        string username = txtUsername.Text;
	        string oldOtc = txtOldOTC.Text;
	        string newOtc = txtNewOTC.Text;
			PINsafeClient client = PINsafeClient.GetWebInstance();
			PINsafeRequest psReq = new PINsafeRequest();
	        if (client.ChangePin(psReq, username, "", "", oldOtc, newOtc)) {
				CheckCookies();
		        string redirect = (String.IsNullOrEmpty(txtRedirect.Value)) ? client.Settings.LogonUrl : txtRedirect.Value;
		        Response.Redirect(redirect);
	        } else {
		        lblResult.Text = psReq.Error;
	        }
        }

		protected void btnCancel_Click(object sender, EventArgs e) {
			string reason = txtReason.Value;
			if ("1".Equals(reason)) {
				// PIN expired: you can't skip.
				lblResult.Text = "Your PIN has expired. You must change it before continuing to OWA";
				return;
			}
			CheckCookies();
			PINsafeClient client = PINsafeClient.GetWebInstance();
			string redirect = (String.IsNullOrEmpty(txtRedirect.Value)) ? client.Settings.LogonUrl : txtRedirect.Value;
			Response.Redirect(redirect);
		}

		protected string GetVersionFolder() {
			PINsafeClient client = PINsafeClient.GetWebInstance();
			return client.Settings.CurrentVersion;
		}
    }
}