﻿using System;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.UI;
using com.swivelsecure.pinsafe;

namespace PINsafeLogin.auth {
    public partial class PINsafeLogon : Page {

        protected void Page_Load(object sender, EventArgs e) {

            bool logonValid = false;

            PINsafeClient client = PINsafeClient.GetWebInstance();
            PINsafeSettings settings = client.Settings;

            string logonPage = settings.LogonUrl;
            string authPage = settings.AuthUrl;
            string destination = settings.TargetUrl;

            // Check that all the required cookies exist.
            HttpCookie pinsafeCookie = Request.Cookies["PINSAFE"];
            HttpCookie sidCookie = Request.Cookies["sessionid"];
            HttpCookie caCookie = Request.Cookies["caCookie"];

            if (pinsafeCookie != null && sidCookie != null && caCookie != null) {
                // Check that the PINsafe cookie is valid.
                if (client.CookieValid(pinsafeCookie.Value)) {
                    logonValid = true;
                } else {
                    Debug.WriteLine("Authentication cookie is not valid");
                }
            }

            string username = Request["username"];
            string password = Request["password"];
            string otc = Request["otc"];
            string attribute = Request["userAttribute"];
            string sFlags = Request["flags"];
            int reqFlags = 0;
            if (!String.IsNullOrEmpty(sFlags)) {
                if (!Int32.TryParse(sFlags, out reqFlags)) {
                    reqFlags = 0;
                }
            }
            int trustLevel = 0;
            string sTrust = Request["trusted"];
            if (!String.IsNullOrEmpty(sTrust)) {
                if (!Int32.TryParse(sTrust, out trustLevel)) {
                    trustLevel = 0;
                }
            }
            string userAgent = Request.UserAgent;
            CookieCollection cookies = null;
            String cookieData = "";
            DateTime expiryTime = DateTime.Now + TimeSpan.FromSeconds(settings.IdleTimeSecs);
            String host = Request.ServerVariables["HTTP_HOST"];

			PINsafeRequest request = new PINsafeRequest();
	        bool pinExpired = false;
			if (!logonValid) {
                bool authRequired = // host.Equals(settings.InternalHost) ||
                                    client.CheckAuthenticationRequired(request, username, Request.UserHostAddress);
                Debug.WriteLine(authRequired ? "Authentication required" : "Authentication NOT required");
                cookieData = authRequired
                                 ? ((attribute == null)
                                        ? client.Login(request, username, "", otc, expiryTime)
                                        : client.LoginByAttribute(request, username, attribute, "", otc, expiryTime))
                                 : client.GetCookie(username, expiryTime);
				// Check for PIN expired.
				pinExpired = (cookieData == null && request.Error != null && request.Error.Equals(PINsafeClient.PIN_EXPIRED));
				if (cookieData != null || pinExpired) {
                    string error;
                    cookies = client.LoginOWA(authPage, username, password, Request.Cookies, host, ref destination,
                                              reqFlags, trustLevel, userAgent, out error);
                    if (cookies != null) {
                        Debug.WriteLine("OWA logon was successful for user \"" + username + "\"");
                        logonValid = true;
                    } else {
                        Debug.WriteLine("OWA logon failed for user \"" + username + "\"");
                    }
                } else {
                    Debug.WriteLine("PINsafe logon failed for user \"" + username + "\"");
                }
            }
            if (logonValid) {
	            int cpReason = 0;
	            if (pinExpired) {
		            cpReason = 1;
	            } else {
		            // Check if PIN is due to expire.
		            String[] warnings = request.Warnings;
		            if (warnings != null && warnings.Length > 0) {
			            foreach (string warning in warnings) {
				            if (warning.Equals(PINsafeClient.PIN_EXPIRES_SOON)) {
					            cpReason = 3;
					            break;
				            }
				            if (warning.Equals(PINsafeClient.PIN_EXPIRES_TODAY)) {
					            cpReason = 2;
								break;
				            }
			            }
		            }
	            }
	            if (cookieData != null) {
		            // Get the cookies.
		            pinsafeCookie = cpReason > 0 ? new HttpCookie("PINSAFE_CP", cookieData) {Path = "/"} : new HttpCookie("PINSAFE", cookieData) {Path = "/"};
		            Response.Cookies.Add(pinsafeCookie);
	            }
	            if (cookies != null) {
                    foreach (
                        HttpCookie newCookie in
                            from Cookie cookie in cookies
                            select new HttpCookie(cookie.Name, cookie.Value) {Expires = cookie.Expires}) {
                        Response.Cookies.Add(newCookie);
                    }
                }
	            if (String.IsNullOrEmpty(destination)) {
		            destination = settings.AuthUrl;
	            }
	            if (cpReason > 0 && settings.RedirectChangePin) {
		            // Redirect to Change PIN.
					Response.Redirect("./ChangePIN.aspx?username=" + username + "&reason=" + cpReason + "&redirect=" + destination);
	            } else {
		            Response.Redirect(destination);
	            }
            } else {
                Response.Redirect(logonPage);
            }
            Response.End();
        }
    }
}